@@ -43,6 +43,9 @@ FileSurveyInput <- function(id, label = "Upload data (csv/xlsx/sav/sas7bdat/dta)
     fileInput(ns("file"), label),
     uiOutput(ns("factor")),
     uiOutput(ns("survey")),
+    uiOutput(ns("binary_check")),
+    uiOutput(ns("binary_var")),
+    uiOutput(ns("binary_val")),
     uiOutput(ns("subset_check")),
     uiOutput(ns("subset_var")),
     uiOutput(ns("subset_val"))
@@ -91,6 +94,8 @@ FileSurveyInput <- function(id, label = "Upload data (csv/xlsx/sav/sas7bdat/dta)
 #' @import shiny
 #' @importFrom data.table fread data.table .SD :=
 #' @importFrom readxl read_excel
+#' @importFrom readr guess_encoding
+#' @importFrom utils read.csv
 #' @importFrom jstable mk.lev
 #' @importFrom haven read_sav read_sas read_dta
 
@@ -98,7 +103,7 @@ FileSurveyInput <- function(id, label = "Upload data (csv/xlsx/sav/sas7bdat/dta)
 FileSurvey <- function(input, output, session, nfactor.limit = 20) {
 
   ## To remove NOTE.
-  variable <- NULL
+  val_label <- BinaryGroupRandom <- variable <- NULL
 
   # The selected file, if any
   userFile <- eventReactive(input$file, {
@@ -116,8 +121,11 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
     validate(need((grepl("csv", userFile()$name) == T) | (grepl("xlsx", userFile()$name) == T) | (grepl("sav", userFile()$name) == T) | (grepl("sas7bdat", userFile()$name) == T), message = "Please upload csv/xlsx/sav/sas7bdat file"))
     if (grepl("csv", userFile()$name) == T){
       out <- data.table::fread(userFile()$datapath, check.names = F, integer64 = "double")
+      if (readr::guess_encoding(userFile()$datapath)[1, 1] == "EUC-KR"){
+        out <- data.table::data.table(utils::read.csv(userFile()$datapath, check.names = F, fileEncoding = "EUC-KR"))
+      }
     } else if (grepl("xlsx", userFile()$name) == T){
-      out = data.table::data.table(readxl::read_excel(userFile()$datapath), check.names = F, integer64 = "double")
+      out <- data.table::data.table(readxl::read_excel(userFile()$datapath), check.names = F, integer64 = "double")
     } else if (grepl("sav", userFile()$name) == T){
       out <- data.table::data.table(tryCatch(haven::read_sav(userFile()$datapath), error = function(e){return(haven::read_sav(userFile()$datapath, encoding = "latin1"))}), check.names = F)
       #out = data.table::data.table(haven::read_sav(userFile()$datapath, encoding = "latin1"), check.names = F, integer64 = "double")
@@ -147,7 +155,7 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
     out <- data.table::data.table(out, check.names = T)
     name.new <- names(out)
     ref <- list(name.old = name.old, name.new = name.new)
-
+    #ref <- data.table::data.table(name.old = name.old, name.new = name.new);setkey(ref, name.new)
 
     #numstart.vnum <- suppressWarnings(sapply(names(out),function(x){!is.na(as.numeric(substr(x, 1,1)))}))
     #names(out)[numstart.vnum] <- paste("n_", names(out)[numstart.vnum], sep = "")
@@ -175,6 +183,7 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
 
 
 
+
   observeEvent(input$factor_vname, {
     output$survey <-  renderUI({
       conti_new <- setdiff(data()$conti_original, input$factor_vname)
@@ -207,16 +216,53 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
         selectInput(session$ns("weights_vname"), label = "Weights",
                     choices = c("None", conti_new), multiple = F,
                     selected = selected.weight.final)
-        )
-      })
+      )
+    })
   })
 
+
   observeEvent(c(data()$factor_original, input$factor_vname, input$repeated_vname, input$cluster_vname, input$strata_vname, input$weights_vname), {
+    output$binary_check <- renderUI({
+      checkboxInput(session$ns("check_binary"), "Make binary variables")
+    })
+
     output$subset_check <- renderUI({
       checkboxInput(session$ns("check_subset"), "Subset data")
     })
+
+    var.conti <- setdiff(names(data()$data), c(data()$factor_original, input$factor_vname, input$repeated_vname, input$cluster_vname, input$strata_vname, input$weights_vname))
+    output$binary_var <- renderUI({
+      req(input$check_binary == T)
+      selectInput(session$ns("var_binary"), "Variables to dichotomize",
+                  choices = var.conti, multiple = T,
+                  selected = var.conti[1])
+    })
+
+    output$binary_val <- renderUI({
+      req(input$check_binary == T)
+      req(length(input$var_binary) > 0)
+      outUI <- tagList()
+      for (v in seq_along(input$var_binary)){
+        med <- stats::quantile(data()$data[[input$var_binary[[v]]]], c(0.05, 0.5, 0.95), na.rm = T)
+        outUI[[v]] <- splitLayout(cellWidths = c("25%", "75%"),
+                                  selectInput(session$ns(paste0("con_binary", v)), paste0("Define reference:"),
+                                              choices = c("\u2264", "\u2265", "\u003c", "\u003e"), selected = "\u2264"
+                                  ),
+                                  numericInput(session$ns(paste0("cut_binary", v)), input$var_binary[[v]],
+                                               value = med[2], min = med[1], max = med[3]
+                                  )
+        )
+
+      }
+      outUI
+
+    })
   })
 
+
+
+
+
   observeEvent(input$check_subset, {
     output$subset_var <- renderUI({
       req(input$check_subset == T)
@@ -227,29 +273,33 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
       #)
 
       tagList(
-        selectInput(session$ns("var_subset"), "Subset variable",
-                    choices = names(data()$data), multiple = F,
+        selectInput(session$ns("var_subset"), "Subset variables",
+                    choices = names(data()$data), multiple = T,
                     selected = names(data()$data)[1])
       )
     })
 
     output$subset_val <- renderUI({
       req(input$check_subset == T)
-      req(input$var_subset)
+      req(length(input$var_subset) > 0)
       var.factor <- c(data()$factor_original, input$factor_vname)
-      var.conti <- setdiff(data()$conti_original, input$factor_vname)
+      outUI <- tagList()
+
+      for (v in seq_along(input$var_subset)){
+        if (input$var_subset[[v]] %in% var.factor){
+          varlevel <- levels(as.factor(data()$data[[input$var_subset[[v]]]]))
+          outUI[[v]] <- selectInput(session$ns(paste0("val_subset", v)), paste0("Subset value: ", input$var_subset[[v]]),
+                                    choices = varlevel, multiple = T,
+                                    selected = varlevel[1])
+        } else{
+          val <- stats::quantile(data()$data[[input$var_subset[[v]]]], na.rm = T)
+          outUI[[v]] <- sliderInput(session$ns(paste0("val_subset", v)), paste0("Subset range: ", input$var_subset[[v]]),
+                                    min = val[1], max = val[5],
+                                    value = c(val[2], val[4]))
+        }
 
-      if (input$var_subset %in% var.factor){
-        varlevel <- levels(as.factor(data()$data[[input$var_subset]]))
-        selectInput(session$ns("val_subset"), "Subset value",
-                    choices = varlevel, multiple = T,
-                    selected = varlevel[1])
-      } else{
-        val <- stats::quantile(data()$data[[input$var_subset]], na.rm = T)
-        sliderInput(session$ns("val_subset"), "Subset range",
-                    min = val[1], max = val[5],
-                    value = c(val[2], val[4]))
       }
+      outUI
     })
   })
 
@@ -263,14 +313,20 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
     cat(msg, "\n")
   })
 
+
   outdata <- reactive({
     req(input$weights_vname)
-    out <- data()$data
+    req(input$cluster_vname)
+    req(input$strata_vname)
+    req(!is.null(input$check_binary))
+
+    out <- data.table::data.table(data()$data)
     out[, (data()$conti_original) := lapply(.SD, function(x){as.numeric(as.vector(x))}), .SDcols = data()$conti_original]
     if (length(input$factor_vname) > 0){
       out[, (input$factor_vname) := lapply(.SD, as.factor), .SDcols= input$factor_vname]
     }
 
+
     ## Make survey object
     if (input$cluster_vname == "None"){
       cluster.survey <- as.formula("~ 1")
@@ -293,60 +349,109 @@ FileSurvey <- function(input, output, session, nfactor.limit = 20) {
       out <- out[!is.na(get(input$weights_vname))]
     }
 
-    #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
-    surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
-                           error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
-
-
     ref <- data()$ref
     out.label <- mk.lev(out)
-    for (vn in ref[["name.new"]]){
-      w <- which(ref[["name.new"]] == vn)
-      out.label[variable == vn, var_label := ref[["name.old"]][w]]
+    #out.label[, var_label := ref[out.label$variable, name.old]]
+
+
+    if (!is.null(input$check_binary) & input$check_binary){
+      validate(
+        need(length(input$var_binary) > 0 , "No variables to dichotomize")
+      )
+      sym.ineq <- c("\u2264", "\u2265", "\u003c", "\u003e")
+      names(sym.ineq) <- sym.ineq[4:1]
+      sym.ineq2 <- c("le", "ge", "l", "g")
+      names(sym.ineq2) <- sym.ineq
+      for (v in seq_along(input$var_binary)){
+        req(input[[paste0("con_binary", v)]])
+        req(input[[paste0("cut_binary", v)]])
+        if (input[[paste0("con_binary", v)]] == "\u2264"){
+          out[, BinaryGroupRandom := factor(1 - as.integer(get(input$var_binary[[v]]) <= input[[paste0("cut_binary", v)]]))]
+
+        } else if (input[[paste0("con_binary", v)]] == "\u2265"){
+          out[, BinaryGroupRandom := factor(1 - as.integer(get(input$var_binary[[v]]) >= input[[paste0("cut_binary", v)]]))]
+        } else if (input[[paste0("con_binary", v)]] == "\u003c"){
+          out[, BinaryGroupRandom := factor(1 - as.integer(get(input$var_binary[[v]]) < input[[paste0("cut_binary", v)]]))]
+        } else{
+          out[, BinaryGroupRandom := factor(1 - as.integer(get(input$var_binary[[v]]) > input[[paste0("cut_binary", v)]]))]
+        }
+        cn.new <- paste0(input$var_binary[[v]], "_group_", sym.ineq2[input[[paste0("con_binary", v)]]], input[[paste0("cut_binary", v)]])
+        data.table::setnames(out, "BinaryGroupRandom", cn.new)
+
+        label.binary <- mk.lev(out[, .SD, .SDcols = cn.new])
+        label.binary[, var_label := paste0(input$var_binary[[v]], "_group")]
+        label.binary[, val_label := paste0(c(input[[paste0("con_binary", v)]], sym.ineq[input[[paste0("con_binary", v)]]]), " ", input[[paste0("cut_binary", v)]])]
+
+        out.label <- rbind(out.label, label.binary)
+      }
+      #surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
+      #                       error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
+
     }
 
+
     if (!is.null(input$check_subset)){
       if (input$check_subset){
         validate(
-          need(length(input$var_subset) > 0 , "No variables for subsetting")
+          need(length(input$var_subset) > 0 , "No variables for subsetting"),
+          need(all(sapply(1:length(input$var_subset), function(x){length(input[[paste0("val_subset", x)]])})), "No value for subsetting")
         )
 
         var.factor <- c(data()$factor_original, input$factor_vname)
         var.conti <- setdiff(data()$conti_original, input$factor_vname)
 
-        if (input$var_subset %in% var.factor){
-          out <- out[get(input$var_subset) %in% input$val_subset]
-          #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
-          surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
-                                 error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
-          #var.factor <- c(data()$factor_original, input$factor_vname)
-          out[, (var.factor) := lapply(.SD, factor), .SDcols = var.factor]
-          out.label2 <- mk.lev(out)[, c("variable", "class", "level")]
-          data.table::setkey(out.label, "variable", "class", "level")
-          data.table::setkey(out.label2, "variable", "class", "level")
-          out.label <- out.label[out.label2]
-        } else{
-          out <- out[get(input$var_subset) >= input$val_subset[1] & get(input$var_subset) <= input$val_subset[2]]
-          #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
-          surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
-                                 error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
+        for (v in seq_along(input$var_subset)){
+          if (input$var_subset[[v]] %in% var.factor){
+            out <- out[get(input$var_subset[[v]]) %in% input[[paste0("val_subset", v)]]]
+            #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
+            #surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
+            #                       error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
+            #var.factor <- c(data()$factor_original, input$factor_vname)
+            out[, (var.factor) := lapply(.SD, factor), .SDcols = var.factor]
+            out.label2 <- mk.lev(out)[, c("variable", "class", "level")]
+            data.table::setkey(out.label, "variable", "class", "level")
+            data.table::setkey(out.label2, "variable", "class", "level")
+            out.label <- out.label[out.label2]
+          } else{
+            out <- out[get(input$var_subset[[v]]) >= input[[paste0("val_subset", v)]][1] & get(input$var_subset[[v]]) <= input[[paste0("val_subset", v)]][2]]
+            #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
+            #surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
+            #                       error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
 
-          #var.factor <- c(data()$factor_original, input$factor_vname)
-          out[, (var.factor) := lapply(.SD, factor), .SDcols = var.factor]
-          out.label2 <- mk.lev(out)[, c("variable", "class", "level")]
-          data.table::setkey(out.label, "variable", "class", "level")
-          data.table::setkey(out.label2, "variable", "class", "level")
-          out.label <- out.label[out.label2]
+            #var.factor <- c(data()$factor_original, input$factor_vname)
+            out[, (var.factor) := lapply(.SD, factor), .SDcols = var.factor]
+            out.label2 <- mk.lev(out)[, c("variable", "class", "level")]
+            data.table::setkey(out.label, "variable", "class", "level")
+            data.table::setkey(out.label2, "variable", "class", "level")
+            out.label <- out.label[out.label2]
+          }
         }
 
       }
     }
 
+    for (vn in ref[["name.new"]]){
+      w <- which(ref[["name.new"]] == vn)
+      out.label[variable == vn, var_label := ref[["name.old"]][w]]
+    }
+
+
+    surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
+                           error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
+
+
     return(list(data = out, label = out.label, naomit = data()$naomit, survey = surveydata))
   })
 
 
 
+    #surveydata <- survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out)
+    #surveydata <- tryCatch(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out),
+    #                       error = function(e){return(survey::svydesign(id = cluster.survey, strata = strata.survey, weights = weights.survey, data = out, nest = T))})
+
+
+
+
   # Return the reactive that yields the data frame
   return(outdata)
 }
